==> fhir_path-0.5.0-13.tar.gz: lib/parser/function_parser/existence_parser.dart <==
import '../../fhir_path.dart';

/// Returns true if the input collection is empty ({ }) and false otherwise.
class EmptyParser extends FhirPathParser {
  EmptyParser();
  List execute(List results, Map passed) => results.isEmpty ? [true] : [false];
}

class HasValueParser extends FhirPathParser {
  HasValueParser();
  late ParserList value;
  List execute(List results, Map passed) {
    // Returns true if the input collection contains a single value which is a FHIR primitive,...
    if (results.length != 1) {
      return [false];
    }

    final element = results.first;

    if (element == null) {
      return [false];
    }

    // ...and it has a primitive value
    // (e.g. as opposed to not having a value and just having extensions).

    if (element is Map<String, dynamic>) {
      // element is a Map, most likely an answer. Introspect further...
      return [
        element.entries.any((mapEntry) =>
            mapEntry.key.startsWith('value') && mapEntry.value != null)
      ];
    } else {
      // element is a Dart primitive
      return [true];
    }
  }
}

/// Returns true if the collection has any elements, and false otherwise.
/// This is the opposite of empty(), and as such is a shorthand for
/// empty().not(). If the input collection is empty ({ }), the result is false.
/// The function can also take an optional criteria to be applied to the
/// collection prior to the determination of the exists. In this case, the
/// function is shorthand for where(criteria).exists().
/// Note that a common term for this function is any.
class ExistsParser extends FunctionParser {
  ExistsParser();
  late ParserList value;
  List execute(List results, Map passed) {
    final returnList = [];
    results.forEach((element) {
      final newResult = value.execute([element], passed);
      if (newResult.isNotEmpty) {
        if (!(newResult.length == 1 && newResult.first == false)) {
          returnList.add(element);
        }
      }
    });
    return [returnList.isNotEmpty];
  }
}

class AllParser extends ValueParser<ParserList> {
  AllParser();
  late ParserList value;
  List execute(List results, Map passed) {
    if (results.isEmpty) {
      return [true];
    }
    final executedValue = value.execute(results.toList(), passed);
    for (var r in executedValue) {
      if (r != true) {
        return [false];
      }
    }
    return [true];
  }
}

/// Takes a collection of Boolean values and returns true if all the items are true.
/// If any items are false, the result is false. If the input is empty ({ }), the result is true.
class AllTrueParser extends FhirPathParser {
  AllTrueParser();
  List execute(List results, Map passed) {
    if (results.isEmpty) {
      return [true];
    }
    results.removeWhere((element) => element == true);
    return [results.isEmpty];
  }
}

/// Takes a collection of Boolean values and returns true if any of the items are true.
/// If all the items are false, or if the input is empty ({ }), the result is false.
class AnyTrueParser extends FhirPathParser {
  AnyTrueParser();
  List execute(List results, Map passed) {
    if (results.isEmpty) {
      return [false];
    }
    results.retainWhere((element) => element == true);
    return [results.length > 0];
  }
}

/// Takes a collection of Boolean values and returns true if all the items are false.
/// If any items are true, the result is false. If the input is empty ({ }), the result is true.
class AllFalseParser extends FhirPathParser {
  AllFalseParser();
  List execute(List results, Map passed) {
    if (results.isEmpty) {
      return [true];
    }
    results.removeWhere((element) => element == false);
    return [results.length == 0];
  }
}

/// Takes a collection of Boolean values and returns true if any of the items are false.
/// If all the items are true, or if the input is empty ({ }), the result is false.
class AnyFalseParser extends FhirPathParser {
  AnyFalseParser();
  List execute(List results, Map passed) {
    if (results.isEmpty) {
      return [false];
    }
    results.retainWhere((element) => element == false);
    return [results.length > 0];
  }
}

class SubsetOfParser extends ValueParser<ParserList> {
  SubsetOfParser();
  late ParserList value;
  List execute(List results, Map passed) {
    if (results.isEmpty) {
      return [true];
    } else {
      final executedValue = value.execute(results.toList(), passed);
      for (var r in results) {
        if (notFoundInList(executedValue, r)) {
          return [false];
        }
      }
      return [true];
    }
  }
}

class SupersetOfParser extends FhirPathParser {
  SupersetOfParser();
  dynamic value;
  List execute(List results, Map passed) {
    if (results.isEmpty) {
      return [false];
    } else {
      final executedValue = value.execute(results.toList(), passed);
      for (var v in executedValue) {
        if (notFoundInList(results, v)) {
          return [false];
        }
      }
      return [true];
    }
  }
}

class CountParser extends FhirPathParser {
  CountParser();
  List execute(List results, Map passed) => [results.length];
}

class DistinctParser extends FhirPathParser {
  DistinctParser();
  List execute(List results, Map passed) {
    final resultsList = [];
    for (var r in results) {
      if (notFoundInList(resultsList, r)) {
        resultsList.add(r);
      }
    }
    return resultsList;
  }
}

class IsDistinctParser extends FhirPathParser {
  IsDistinctParser();
  List execute(List results, Map passed) {
    final resultsList = [];
    for (var r in results) {
      if (notFoundInList(resultsList, r)) {
        resultsList.add(r);
      }
    }
    return [resultsList.length == results.length];
  }
}

==> fhir_path-0.5.0-19.tar.gz: lib/parser/function_parser/existence_parser.dart <==
import '../../fhir_path.dart';

/// Returns true if the input collection is empty ({ }) and false otherwise.
class EmptyParser extends FhirPathParser {
  EmptyParser();
  List execute(List results, Map<String, dynamic> passed) =>
      results.isEmpty ? [true] : [false];
}

class HasValueParser extends FhirPathParser {
  HasValueParser();
  late ParserList value;
  List execute(List results, Map<String, dynamic> passed) {
    // Returns true if the input collection contains a single value which is a FHIR primitive,...
    if (results.length != 1) {
      return [false];
    }

    final element = results.first;

    if (element == null) {
      return [false];
    }

    // ...and it has a primitive value
    // (e.g. as opposed to not having a value and just having extensions).

    if (element is Map<String, dynamic>) {
      // element is a Map, most likely an answer. Introspect further...
      return [
        element.entries.any((mapEntry) =>
            mapEntry.key.startsWith('value') && mapEntry.value != null)
      ];
    } else {
      // element is a Dart primitive
      return [true];
    }
  }
}

/// Returns true if the collection has any elements, and false otherwise.
/// This is the opposite of empty(), and as such is a shorthand for
/// empty().not(). If the input collection is empty ({ }), the result is false.
/// The function can also take an optional criteria to be applied to the
/// collection prior to the determination of the exists. In this case, the
/// function is shorthand for where(criteria).exists().
/// Note that a common term for this function is any.
class ExistsParser extends FunctionParser {
  ExistsParser();
  late ParserList value;
  List execute(List results, Map<String, dynamic> passed) {
    final returnList = [];
    results.forEach((element) {
      final newResult = value.execute([element], passed);
      if (newResult.isNotEmpty) {
        if (!(newResult.length == 1 && newResult.first == false)) {
          returnList.add(element);
        }
      }
    });
    return [returnList.isNotEmpty];
  }
}

class AllParser extends ValueParser<ParserList> {
  AllParser();
  late ParserList value;
  List execute(List results, Map<String, dynamic> passed) {
    if (results.isEmpty) {
      return [true];
    }
    final executedValue = value.execute(results.toList(), passed);
    for (var r in executedValue) {
      if (r != true) {
        return [false];
      }
    }
    return [true];
  }
}

/// Takes a collection of Boolean values and returns true if all the items are true.
/// If any items are false, the result is false. If the input is empty ({ }), the result is true.
class AllTrueParser extends FhirPathParser {
  AllTrueParser();
  List execute(List results, Map<String, dynamic> passed) {
    if (results.isEmpty) {
      return [true];
    }
    results.removeWhere((element) => element == true);
    return [results.isEmpty];
  }
}

/// Takes a collection of Boolean values and returns true if any of the items are true.
/// If all the items are false, or if the input is empty ({ }), the result is false.
class AnyTrueParser extends FhirPathParser {
  AnyTrueParser();
  List execute(List results, Map<String, dynamic> passed) {
    if (results.isEmpty) {
      return [false];
    }
    results.retainWhere((element) => element == true);
    return [results.length > 0];
  }
}

/// Takes a collection of Boolean values and returns true if all the items are false.
/// If any items are true, the result is false. If the input is empty ({ }), the result is true.
class AllFalseParser extends FhirPathParser {
  AllFalseParser();
  List execute(List results, Map<String, dynamic> passed) {
    if (results.isEmpty) {
      return [true];
    }
    results.removeWhere((element) => element == false);
    return [results.length == 0];
  }
}

/// Takes a collection of Boolean values and returns true if any of the items are false.
/// If all the items are true, or if the input is empty ({ }), the result is false.
class AnyFalseParser extends FhirPathParser {
  AnyFalseParser();
  List execute(List results, Map<String, dynamic> passed) {
    if (results.isEmpty) {
      return [false];
    }
    results.retainWhere((element) => element == false);
    return [results.length > 0];
  }
}

class SubsetOfParser extends ValueParser<ParserList> {
  SubsetOfParser();
  late ParserList value;
  List execute(List results, Map<String, dynamic> passed) {
    if (results.isEmpty) {
      return [true];
    } else {
      final executedValue = value.execute(results.toList(), passed);
      for (var r in results) {
        if (notFoundInList(executedValue, r)) {
          return [false];
        }
      }
      return [true];
    }
  }
}

class SupersetOfParser extends FhirPathParser {
  SupersetOfParser();
  dynamic value;
  List execute(List results, Map<String, dynamic> passed) {
    if (results.isEmpty) {
      return [false];
    } else {
      final executedValue = value.execute(results.toList(), passed);
      for (var v in executedValue) {
        if (notFoundInList(results, v)) {
          return [false];
        }
      }
      return [true];
    }
  }
}

class CountParser extends FhirPathParser {
  CountParser();
  List execute(List results, Map<String, dynamic> passed) => [results.length];
}

class DistinctParser extends FhirPathParser {
  DistinctParser();
  List execute(List results, Map<String, dynamic> passed) {
    final resultsList = [];
    for (var r in results) {
      if (notFoundInList(resultsList, r)) {
        resultsList.add(r);
      }
    }
    return resultsList;
  }
}

class IsDistinctParser extends FhirPathParser {
  IsDistinctParser();
  List execute(List results, Map<String, dynamic> passed) {
    final resultsList = [];
    for (var r in results) {
      if (notFoundInList(resultsList, r)) {
        resultsList.add(r);
      }
    }
    return [resultsList.length == results.length];
  }
}
